<?php

/**
* 10/08/2018
* @author Fabio Pereira <fabio.pereira@teknisa.com>
*/

namespace MYO\Util;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response;

class MessageProcedure {

    protected $entityManager;
	
	const LENGTH_CODE_MSG 		= 12;
	const SQL_LISTAERROS        = "SELECT DSERRO, MIN(NRORDEM) AS NRORDEM FROM LISTAERROS WHERE CDFILIAL = :CDFILIAL AND NRLISTA = :NRLISTA GROUP BY DSERRO ORDER BY NRORDEM";
	const SQL_DELETE_LISTAERROS = "DELETE FROM LISTAERROS WHERE CDFILIAL = :CDFILIAL AND NRLISTA = :NRLISTA";

	const FILE_FORMAT_DEFAULT = '.txt';

    const SEPARATORTOSTART  = "APPERRMSGSTR";
    const SEPARATORTOEND    = "APPERRMSGEND";
	const SEPARATORKEY      = "__";
	const SEPARATORDELIMIT  = "$$";

	const MESSAGESBYCODE = array(
        //MyOrder
        "MYO_SERVREQEX" => "Pedido: __0__ - O serviço __1__ não está habilitado para gerar requisição para este operador.",
        "MYO_REQEXRET" => "Pedido: __0__ - O serviço__1__ possui a requisição __2__ com retirada lançada, portanto não será re-calculada."
	);

    public function __construct ($entityManager) {
        $this->entityManager = $entityManager;
    }

	public function concatErrorMessages($cdfilial, $nrlista, $response, $header = null, $returnToResponse = true, $downloadFileError = false){
		$params = array ("CDFILIAL" => $cdfilial, "NRLISTA"  => $nrlista);

		$query = $this->entityManager->getConnection()->prepare(self::SQL_LISTAERROS);
        $query->execute($params);
        $result = $query->fetchAll();
        $erros = array();
        if($header){
        	array_push($erros, $header);
        }

        for ($i = 0; $i < count($result); $i++) {
		    $resultado = $this->convertMessage($result[$i]['DSERRO']);
		    array_push($erros, $resultado);
		};

		$queryDelete = $this->entityManager->getConnection()->prepare(self::SQL_DELETE_LISTAERROS);
        $queryDelete->execute($params);

        $dataSets = $response->getDataSets();
        $listaErros;
        foreach($dataSets as $dataSet) {
        	if($dataSet->getDataSourceName() == 'ListaErros'){
        		$listaErros = $dataSet;
        		break;
        	}
        }
        if(isset($listaErros)) {
        	$oldRows = $listaErros->getRows();
        	$erros = array_merge($erros, $oldRows);
		}

		if($downloadFileError){
			//Download file with list error
			$fileName = 'LISTA_ERROS_'.$cdfilial.'_'.$nrlista;
			$fileError = $this->createFile($erros);
			$this->downloadFile($fileName, $fileError);
		} else {
			if($returnToResponse){
				return $response->addDataSet(new DataSet("ListaErros", $erros));
			} else {
				return $erros;
			}
		}
	}

	public function convertMessage($message){
		$messageUpper 	= strtoupper($message);
		$codeStart	 	= strpos($messageUpper, self::SEPARATORTOSTART);
		$codeEnd		= strpos($messageUpper, self::SEPARATORTOEND);

		if ($codeStart !== FALSE && $codeEnd !== FALSE){
			$msgStart = $codeStart + self::LENGTH_CODE_MSG;
			$msgEnd = $codeEnd - $msgStart;

	        $paramstring = substr($message, $msgStart, $msgEnd);

	        $param = explode(self::SEPARATORDELIMIT, $paramstring);

	        $code = array_shift($param);

	        $messageConvert = $this->getMessage($code, $param);
	        $messageConvert = $messageConvert ? $messageConvert : $message;

	        return $messageConvert;
		}
		return $message;
	}

	public function getMessage($code, $param) {

		if(array_key_exists($code, self::MESSAGESBYCODE)){
			return $this->replaceValues(self::MESSAGESBYCODE[$code], $param);
		} else {
			//Did not find a message
			return false;
		}
	}

	private function replaceValues($msg, $params) {
		if (is_array($params)) {
			foreach(array_keys($params) as $key){
				$msg = str_replace($this->getKeyName($key), $params[$key], $msg);
			}
		}
		return $msg;
	}

	private function getKeyName($key) {
		return self::SEPARATORKEY.$key.self::SEPARATORKEY;
	}

	public function createFile($erros){
		$erros = is_array($erros) ? $erros : array($erros);
		$file = null;
		$file .= "\t\t  Lista de Erros\r\n";
		foreach ($erros as $msgError) {
			$file .= $msgError."\r\n";
		}
		return $file;
	}

	public function downloadFile($filename, $outFile) {
		$fileContent = "";
		header('Accept-Ranges: none');
		header('Content-Description: File Transfer');
		header('Content-Type: application/octet-stream');
		header('Content-Transfer-Encoding: binary');
		header('Connection: Keep-Alive');
		header('Expires: 0');
		header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
		header('Pragma: public');
		header("Content-Disposition: attachment; filename=".$filename.self::FILE_FORMAT_DEFAULT);
		echo $outFile;
		exit;
	}
}